require 'spec_helper'

describe 'when skip_if condition is defined' do
  describe '#previous_step' do
    context 'when second step' do
      context 'when should NOT be skipped' do
        before(:all) {
          @controller = SkipIf::StepTwoController.new
        }

        it 'should get step 1' do
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_two?).returns(false)
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_one?).returns(false)
          get :show

          expect(subject.previous_step.label).to eq 'Step 1'
        end
      end

      context 'when should be skipped' do
        before(:all) {
          @controller = SkipIf::StepTwoController.new
        }

        it 'should return nil' do
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_two?).returns(false)
          get :show

          expect(subject.previous_step).to be_nil
        end
      end
    end

    context 'when third step' do
      context 'when should NOT be skipped' do
        before(:all) {
          @controller = SkipIf::StepThreeController.new
        }

        it 'should get step 2' do
          SkipIf::StepThreeController.any_instance.stubs(:skip_step_two?).returns(false)
          SkipIf::StepThreeController.any_instance.stubs(:skip_step_three?).returns(false)
          get :show

          expect(subject.previous_step.label).to eq 'Step 2'
        end
      end

      context 'when should be skipped' do
        before(:all) {
          @controller = SkipIf::StepThreeController.new
        }

        it 'should return step 1' do
          SkipIf::StepThreeController.any_instance.stubs(:skip_step_one?).returns(false)
          SkipIf::StepThreeController.any_instance.stubs(:skip_step_three?).returns(false)
          get :show

          expect(subject.previous_step.label).to eq 'Step 1'
        end
      end
    end
  end

  describe '#previous_step_index' do
    context 'when second step' do
      context 'when should NOT be skipped' do
        before(:all) {
          @controller = SkipIf::StepTwoController.new
        }

        it 'should get step 1 index' do
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_two?).returns(false)
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_one?).returns(false)
          get :show

          expect(subject.previous_step_index).to eq 0
        end
      end

      context 'when should be skipped' do
        before(:all) {
          @controller = SkipIf::StepTwoController.new
        }

        it 'should return nil' do
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_two?).returns(false)
          get :show

          expect(subject.previous_step_index).to be_nil
        end
      end
    end

    context 'when third step' do
      context 'when should NOT be skipped' do
        before(:all) {
          @controller = SkipIf::StepThreeController.new
        }

        it 'should get step 2 index' do
          SkipIf::StepThreeController.any_instance.stubs(:skip_step_two?).returns(false)
          SkipIf::StepThreeController.any_instance.stubs(:skip_step_three?).returns(false)
          get :show

          expect(subject.previous_step_index).to eq 1
        end
      end

      context 'when should be skipped' do
        before(:all) {
          @controller = SkipIf::StepThreeController.new
        }

        it 'should return step 1 index' do
          SkipIf::StepThreeController.any_instance.stubs(:skip_step_one?).returns(false)
          SkipIf::StepThreeController.any_instance.stubs(:skip_step_three?).returns(false)
          get :show

          expect(subject.previous_step_index).to eq 0
        end
      end
    end
  end

  describe '#previous_step_name' do
    context 'when first step' do
      before(:all) {
        @controller = SkipIf::StepOneController.new
      }

      it 'should return nil' do
        SkipIf::StepOneController.any_instance.stubs(:skip_step_one?).returns(false)
        get :show

        expect(subject.previous_step_name).to be_nil
      end
    end

    context 'when second step' do
      context 'when should NOT be skipped' do
        before(:all) {
          @controller = SkipIf::StepTwoController.new
        }

        it 'should get step 1 name' do
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_two?).returns(false)
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_one?).returns(false)
          get :show

          expect(subject.previous_step_name).to eq :step_one
        end
      end

      context 'when should be skipped' do
        before(:all) {
          @controller = SkipIf::StepTwoController.new
        }

        it 'should return nil' do
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_two?).returns(false)
          get :show

          expect(subject.previous_step_name).to be_nil
        end
      end
    end

    context 'when third step' do
      context 'when should NOT be skipped' do
        before(:all) {
          @controller = SkipIf::StepThreeController.new
        }

        it 'should get step 2 name' do
          SkipIf::StepThreeController.any_instance.stubs(:skip_step_two?).returns(false)
          SkipIf::StepThreeController.any_instance.stubs(:skip_step_three?).returns(false)
          get :show

          expect(subject.previous_step_name).to eq :step_two
        end
      end

      context 'when should be skipped' do
        before(:all) {
          @controller = SkipIf::StepThreeController.new
        }

        it 'should return step 1 name' do
          SkipIf::StepThreeController.any_instance.stubs(:skip_step_one?).returns(false)
          SkipIf::StepThreeController.any_instance.stubs(:skip_step_three?).returns(false)
          get :show

          expect(subject.previous_step_name).to eq :step_one
        end
      end
    end
  end

  describe '#next_step' do
    context 'when first step' do
      context 'when should NOT be skipped' do
        before(:all) {
          @controller = SkipIf::StepOneController.new
        }

        it 'should return step 2' do
          SkipIf::StepOneController.any_instance.stubs(:skip_step_one?).returns(false)
          SkipIf::StepOneController.any_instance.stubs(:skip_step_two?).returns(false)
          get :show

          expect(subject.next_step.label).to eq 'Step 2'
        end
      end

      context 'when should be skipped' do
        before(:all) {
          @controller = SkipIf::StepOneController.new
        }

        it 'should return step 3' do
          SkipIf::StepOneController.any_instance.stubs(:skip_step_one?).returns(false)
          SkipIf::StepOneController.any_instance.stubs(:skip_step_three?).returns(false)
          get :show

          expect(subject.next_step.label).to eq 'Step 3'
        end
      end
    end

    context 'when second step' do
      context 'when should NOT be skipped' do
        before(:all) {
          @controller = SkipIf::StepTwoController.new
        }

        it 'should get step 3' do
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_two?).returns(false)
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_three?).returns(false)
          get :show

          expect(subject.next_step.label).to eq 'Step 3'
        end
      end

      context 'when should be skipped' do
        before(:all) {
          @controller = SkipIf::StepTwoController.new
        }

        it 'should return nil' do
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_two?).returns(false)
          get :show

          expect(subject.next_step).to be_nil
        end
      end
    end
  end

  describe '#next_step_index' do
    context 'when first step' do
      context 'when should NOT be skipped' do
        before(:all) {
          @controller = SkipIf::StepOneController.new
        }

        it 'should return step 2 index' do
          SkipIf::StepOneController.any_instance.stubs(:skip_step_one?).returns(false)
          SkipIf::StepOneController.any_instance.stubs(:skip_step_two?).returns(false)
          get :show

          expect(subject.next_step_index).to eq 1
        end
      end

      context 'when should be skipped' do
        before(:all) {
          @controller = SkipIf::StepOneController.new
        }

        it 'should return step 3 index' do
          SkipIf::StepOneController.any_instance.stubs(:skip_step_one?).returns(false)
          SkipIf::StepOneController.any_instance.stubs(:skip_step_three?).returns(false)
          get :show

          expect(subject.next_step_index).to eq 2
        end
      end
    end

    context 'when second step' do
      context 'when should NOT be skipped' do
        before(:all) {
          @controller = SkipIf::StepTwoController.new
        }

        it 'should get step 3 index' do
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_two?).returns(false)
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_three?).returns(false)
          get :show

          expect(subject.next_step_index).to eq 2
        end
      end

      context 'when should be skipped' do
        before(:all) {
          @controller = SkipIf::StepTwoController.new
        }

        it 'should return nil' do
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_two?).returns(false)
          get :show

          expect(subject.next_step_index).to be_nil
        end
      end
    end
  end

  describe '#next_step_name' do
    context 'when first step' do
      context 'when should NOT be skipped' do
        before(:all) {
          @controller = SkipIf::StepOneController.new
        }

        it 'should return step_two' do
          SkipIf::StepOneController.any_instance.stubs(:skip_step_one?).returns(false)
          SkipIf::StepOneController.any_instance.stubs(:skip_step_two?).returns(false)
          get :show

          expect(subject.next_step_name).to eq :step_two
        end
      end

      context 'when should be skipped' do
        before(:all) {
          @controller = SkipIf::StepOneController.new
        }

        it 'should return step_three' do
          SkipIf::StepOneController.any_instance.stubs(:skip_step_one?).returns(false)
          SkipIf::StepOneController.any_instance.stubs(:skip_step_three?).returns(false)
          get :show

          expect(subject.next_step_name).to eq :step_three
        end
      end
    end

    context 'when second step' do
      context 'when should NOT be skipped' do
        before(:all) {
          @controller = SkipIf::StepTwoController.new
        }

        it 'should return step_three' do
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_two?).returns(false)
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_three?).returns(false)
          get :show

          expect(subject.next_step_name).to eq :step_three
        end
      end

      context 'when should be skipped' do
        before(:all) {
          @controller = SkipIf::StepTwoController.new
        }

        it 'should return nil' do
          SkipIf::StepTwoController.any_instance.stubs(:skip_step_two?).returns(false)
          get :show

          expect(subject.next_step_name).to be_nil
        end
      end
    end

    context 'when third step' do
      before(:all) {
        @controller = SkipIf::StepThreeController.new
      }

      it 'should return nil' do
        SkipIf::StepThreeController.any_instance.stubs(:skip_step_three?).returns(false)
        get :show

        expect(subject.next_step_name).to be_nil
      end
    end
  end
end