require 'spec_helper'

describe 'when skip_unless condition is defined' do
  describe '#previous_step' do
    context 'when first step' do
      before(:all) {
        @controller = SkipUnless::StepOneController.new
      }

      it 'should be nil' do
        SkipUnless::StepOneController.any_instance.stubs(:dont_skip_step_one?).returns(true)
        get :show

        expect(subject.previous_step).to be_nil
      end
    end

    context 'when second step' do
      context 'when should NOT be skipped' do
        before(:all) {
          @controller = SkipUnless::StepTwoController.new
        }

        it 'should get step 1' do
          SkipUnless::StepTwoController.any_instance.stubs(:dont_skip_step_one?).returns(true)
          SkipUnless::StepTwoController.any_instance.stubs(:dont_skip_step_two?).returns(true)
          get :show

          expect(subject.previous_step.label).to eq 'Step 1'
        end
      end

      context 'when should be skipped' do
        before(:all) {
          @controller = SkipUnless::StepTwoController.new
        }

        it 'should return nil' do
          SkipUnless::StepTwoController.any_instance.stubs(:dont_skip_step_two?).returns(true)
          get :show

          expect(subject.previous_step).to be_nil
        end
      end
    end

    context 'when third step' do
      context 'when should NOT be skipped' do
        before(:all) {
          @controller = SkipUnless::StepThreeController.new
        }

        it 'should get step 2' do
          SkipUnless::StepThreeController.any_instance.stubs(:dont_skip_step_two?).returns(true)
          SkipUnless::StepThreeController.any_instance.stubs(:dont_skip_step_three?).returns(true)
          get :show

          expect(subject.previous_step.label).to eq 'Step 2'
        end
      end

      context 'when should be skipped' do
        before(:all) {
          @controller = SkipUnless::StepThreeController.new
        }

        it 'should be step 1' do
          SkipUnless::StepThreeController.any_instance.stubs(:dont_skip_step_one?).returns(true)
          SkipUnless::StepThreeController.any_instance.stubs(:dont_skip_step_three?).returns(true)
          get :show

          expect(subject.previous_step.label).to eq 'Step 1'
        end
      end
    end
  end

  describe '#next_step' do
    context 'when first step' do
      context 'when should NOT be skipped' do
        before(:all) {
          @controller = SkipUnless::StepOneController.new
        }

        it 'should get step 2' do
          SkipUnless::StepOneController.any_instance.stubs(:dont_skip_step_one?).returns(true)
          SkipUnless::StepOneController.any_instance.stubs(:dont_skip_step_two?).returns(true)
          get :show

          expect(subject.next_step.label).to eq 'Step 2'
        end
      end

      context 'when should be skipped' do
        before(:all) {
          @controller = SkipUnless::StepOneController.new
        }

        it 'should get step 3' do
          SkipUnless::StepOneController.any_instance.stubs(:dont_skip_step_one?).returns(true)
          SkipUnless::StepOneController.any_instance.stubs(:dont_skip_step_three?).returns(true)
          get :show

          expect(subject.next_step.label).to eq 'Step 3'
        end
      end
    end

    context 'when second step' do
      context 'when should NOT be skipped' do
        before(:all) {
          @controller = SkipUnless::StepTwoController.new
        }

        it 'should be step 3' do
          SkipUnless::StepTwoController.any_instance.stubs(:dont_skip_step_two?).returns(true)
          SkipUnless::StepTwoController.any_instance.stubs(:dont_skip_step_three?).returns(true)
          get :show

          expect(subject.next_step.label).to eq 'Step 3'
        end
      end

      context 'when should be skipped' do
        before(:all) {
          @controller = SkipUnless::StepTwoController.new
        }

        it 'should return nil' do
          SkipUnless::StepTwoController.any_instance.stubs(:dont_skip_step_two?).returns(true)
          get :show

          expect(subject.next_step).to be_nil
        end
      end
    end

    context 'when third step' do
      before(:all) {
        @controller = SkipUnless::StepThreeController.new
      }

      it 'should get step 2' do
        SkipUnless::StepThreeController.any_instance.stubs(:dont_skip_step_three?).returns(true)
        get :show

        expect(subject.next_step).to be_nil
      end
    end
  end
end